function [p_low,p_upp,e_low,e_upp]= MAXNSGP(S,n,m,u,f,r,Ti,Tf,a)
%
%  Computes approximate upper and lower bounds for the 
%   maximum of a non centered non stationary Gaussian process P( M_T > u ) and P (|M_T| > u).
%  Inputs: 
%   S =  number of MC replications in the  MCQMC integration method. Typical value : 12;
%   n = number of discretization points on  [Ti,Tf];
%   m =  number of QMC  points in MCQMC integration; Typical value 1000-10000;
%   u =  level; 
%   f =  User defined function f(t)  giving  the mean of  the process. Must admit a vector argument ; 
%   r = User defined function r(s,t)  giving  the covariance of  the process. Must admit a vector argument;
%   Ti, Tf = limits for the time intervaln. Ti<Tf;
%   a: if a = 0, the program computes P( M_T > u ), if a ~= 0, it computes P( |M_T| > u)
%  Outputs: 
%   p_low = approximate lower bound, with
%   e_low = error estimate (MCQMC);
%   p_upp = approximate upper bound, with
%   e_upp = estimate for total error; 
%
%  Example:
% [p_low,p_upp,e_low,e_upp] = MAXNSGP(12,10,5000,2,@(t) 0*t,@(t,s) exp(-(t-s).^2/2),0,5)
% [p_low,p_upp,e_low,e_upp] = MAXNSGP(12,10,5000,5,@(t) sqrt(t),@(t,s) (exp(s.*t) - 1)./sqrt(exp(s.^2)-1)./sqrt(exp(t.^2)-1),1,3)
%
%
%          Stephane Mourareau is the author of this program. MAXNSGP is based on Alan Genz's program MGP.
%          Stephane Mourareau, Institut Mathématiques de Toulouse, Université Paul Sabatier, 31062 Toulouse Cedex 9
%          Email: stephane.mourareau@math.univ-toulouse.fr
%        
% 




if nargin == 8,
    a = 0;
end

h = (Tf - Ti)/n;

if a==0,
    [pv,ev,Sg1,Exp1,p_S0] = upper_bound(S,m,u,n,f,r,Ti,Tf);
    po = 1 - Phi2((u-f(Ti))/sqrt(r(Ti,Ti))); 
    p_In2 = po + h*( p_S0 + 2*sum(pv(2:2:n-2)) + pv(n) ); %p0 + I_{n/2}
    p_In = po + h*( p_S0/2 + sum(pv(1:n-1)) + pv(n)/2 ); %p0 + I_n
    p_upp = ( 4*p_In - p_In2 )/3; %upper bound from Trapezoidal rule
    e_tra = abs( p_In - p_In2 )/3; %error from Trapezoidal rule
    e_QMC = h*sqrt( sum(ev(1:n-1).^2) + ev(n).^2/4 ); %error from MCQMC
    e_upp = e_tra + e_QMC; %total error
    [p_low,e_low] = lower_bound(S,Sg1,Exp1,n,m,u);
else
    [pv1,ev1,pv2,ev2,Sg1,Exp1,p_S0,p_S01] = upper_bound_2(S,m,u,n,f,r,Ti,Tf);
    po = 1 - Phi2((u-f(Ti))/sqrt(r(Ti,Ti))) + Phi2((-u-f(Ti))/sqrt(r(Ti,Ti)));
    p_Jn2 = h*( p_S0 + 2*sum(pv1(2:2:n-2)) + pv1(n) ); %p0 + I_{n/2}
    p_Jn = h*( p_S0/2 + sum(pv1(1:n-1)) + pv1(n)/2 ); %p0 + I_n
    p_upp1 = ( 4*p_Jn - p_Jn2 )/3; %upper bound for Trapezoidal rule
    p_Kn2 = h*( p_S01 + 2*sum(pv2(2:2:n-2)) + pv2(n) ); %p0 + I_{n/2}
    p_Kn = h*( p_S01/2 + sum(pv2(1:n-1)) + pv2(n)/2 ); %p0 + I_n
    p_upp2 = ( 4*p_Kn - p_Kn2 )/3; %uppper bound for Trapezoidal rule
    e_tra1 = abs( p_Jn - p_Jn2 )/3; %error from Trapezoidal rule
    e_tra2 = abs( p_Kn - p_Kn2 )/3; %error from Trapezoidal rule
    e_QMC = h*sqrt( sum(ev1(1:n-1).^2) + ev1(n).^2/4 + sum(ev2(1:n-1).^2) + ev2(n).^2/4); %error from MCQMC
    p_upp = p_upp1 + p_upp2 + po;
    e_upp = e_tra1 + e_tra2 + e_QMC; %total error
    [p_low,e_low] = lower_bound_2(S,Sg1,Exp1,n,m,u);
end

end
